@@ -5,6 +5,7 @@
 -- @since 2.2.8
 module Database.Esqueleto.MySQL
     ( random_
+    , lockInShareMode
     ) where
 
 import Database.Esqueleto.Internal.Internal hiding (random_)
@@ -13,6 +14,18 @@ import Database.Esqueleto.Internal.PersistentImport
 -- | (@random()@) Split out into database specific modules
 -- because MySQL uses `rand()`.
 --
--- /Since: 2.6.0/
+-- @since 2.6.0
 random_ :: (PersistField a, Num a) => SqlExpr (Value a)
 random_ = unsafeSqlValue "RAND()"
+
+-- | @LOCK IN SHARE MODE@ syntax.
+--
+-- Example:
+--
+-- @
+--  'locking' 'lockInShareMode'
+-- @
+--
+-- @since 3.6.0.0
+lockInShareMode :: LockingKind
+lockInShareMode = LockInShareMode
