@@ -21,21 +21,14 @@
 module Common.Record (testDeriveEsqueletoRecord) where
 
 import Common.Test.Import hiding (from, on)
-import Control.Monad.Trans.State.Strict (StateT(..), evalStateT)
-import Data.Bifunctor (first)
 import Data.List (sortOn)
-import Data.Maybe (catMaybes)
-import Data.Proxy (Proxy(..))
 import Database.Esqueleto.Experimental
-import Database.Esqueleto.Internal.Internal (SqlSelect(..))
-import Database.Esqueleto.Record (
-  DeriveEsqueletoRecordSettings(..),
-  defaultDeriveEsqueletoRecordSettings,
-  deriveEsqueletoRecord,
-  deriveEsqueletoRecordWith,
-  takeColumns,
-  takeMaybeColumns,
- )
+import Database.Esqueleto.Record
+       ( DeriveEsqueletoRecordSettings(..)
+       , defaultDeriveEsqueletoRecordSettings
+       , deriveEsqueletoRecord
+       , deriveEsqueletoRecordWith
+       )
 import GHC.Records
 
 data MyRecord =
@@ -67,10 +60,16 @@ myRecordQuery = do
 data MyNestedRecord = MyNestedRecord
   { myName :: Text
   , myRecord :: MyRecord
+  , myMaybeRecord :: Maybe MyRecord
   }
   deriving (Show, Eq)
 
+data MyNestedMaybeRecord = MyNestedMaybeRecord
+  {myNestedRecord :: Maybe MyRecord}
+  deriving (Show, Eq)
+
 $(deriveEsqueletoRecord ''MyNestedRecord)
+$(deriveEsqueletoRecord ''MyNestedMaybeRecord)
 
 myNestedRecordQuery :: SqlQuery SqlMyNestedRecord
 myNestedRecordQuery = do
@@ -89,6 +88,32 @@ myNestedRecordQuery = do
             , myUser = user
             , myAddress = address
             }
+      , myMaybeRecord =
+          SqlMaybeMyRecord
+            { myName = castString $ user ^. #name
+            , myAge = val $ Just 10
+            , myUser = toMaybe user
+            , myAddress = address
+            }
+      }
+
+myNestedMaybeRecordQuery :: SqlQuery SqlMyNestedMaybeRecord
+myNestedMaybeRecordQuery = do
+  user :& address <-
+    from $
+      table @User
+        `leftJoin` table @Address
+        `on` (do \(user :& address) -> user ^. #address ==. address ?. #id)
+  pure
+    SqlMyNestedMaybeRecord
+      {
+        myNestedRecord =
+          SqlMaybeMyRecord
+            { myName = castString $ user ^. #name
+            , myAge = val $ Just 10
+            , myUser = toMaybe user
+            , myAddress = address
+            }
       }
 
 data MyModifiedRecord =
@@ -198,6 +223,107 @@ testDeriveEsqueletoRecord = describe "deriveEsqueletoRecord" $ do
                    } -> addr1 == addr2 -- The keys should match.
                  _ -> False)
 
+    itDb "can select nested maybe records" $ do
+        setup
+        records <- select myNestedMaybeRecordQuery
+        let sortedRecords = sortOn (\MyNestedMaybeRecord {myNestedRecord} -> case myNestedRecord of
+                                       Just r -> getField @"myName" r
+                                       Nothing -> "No name"
+                                    ) records
+        liftIO $ sortedRecords !! 0
+          `shouldSatisfy`
+          (\case MyNestedMaybeRecord
+                   {
+                    myNestedRecord = Just
+                       MyRecord { myName = "Rebecca"
+                                , myAge = Just 10
+                                , myUser = Entity _ User { userAddress  = Nothing
+                                                         , userName = "Rebecca"
+                                                         }
+                                , myAddress = Nothing
+                                }
+                   } -> True
+                 _ -> False)
+
+        liftIO $ sortedRecords !! 1
+          `shouldSatisfy`
+          (\case MyNestedMaybeRecord
+                   {
+                    myNestedRecord = Just
+                       MyRecord { myName = "Some Guy"
+                                , myAge = Just 10
+                                , myUser = Entity _ User { userAddress  = Just addr1
+                                                         , userName = "Some Guy"
+                                                         }
+                                , myAddress = Just (Entity addr2 Address {addressAddress = "30-50 Feral Hogs Rd"})
+                                }
+                   } -> addr1 == addr2 -- The keys should match.
+                 _ -> False)
+
+    itDb "can select nested nothing records" $ do
+        setup
+        records <- select $ do
+          user :& address <-
+            from $ table @User `leftJoin` table @Address `on` (do \(_ :& _) -> val False)
+          pure
+            SqlMyNestedMaybeRecord
+            {
+              myNestedRecord =
+                SqlMaybeMyRecord
+                  { myName = val Nothing
+                  , myAge = val Nothing
+                  , myUser = toMaybe user
+                  , myAddress = address
+                  }
+            }
+        liftIO $ records `shouldBe`
+          [MyNestedMaybeRecord { myNestedRecord = Nothing }, MyNestedMaybeRecord { myNestedRecord = Nothing}]
+
+    itDb "can left join on nested maybed records" $ do
+        setup
+        records <- select $ do
+          from
+            ( table @User
+                `leftJoin` myNestedMaybeRecordQuery
+                `on` (do \(user :& record) -> just (user ^. #id) ==. getField @"myUser" (getField @"myNestedRecord" record) ?. #id)
+            )
+        let sortedRecords = sortOn (\(Entity _ user :& _) -> getField @"userName" user) records
+        liftIO $ sortedRecords !! 0
+          `shouldSatisfy`
+          (\case (_ :& Just (MyNestedMaybeRecord
+                              {
+                                myNestedRecord = Just
+                                  MyRecord { myName = "Rebecca",
+                                             myAddress = Nothing
+                                           }
+                              }
+                            )) -> True
+                 _ -> False)
+        liftIO $ sortedRecords !! 1
+          `shouldSatisfy`
+          (\case ( _ :& Just (MyNestedMaybeRecord
+                               { myNestedRecord = Just
+                                   MyRecord { myName = "Some Guy"
+                                            , myAddress = (Just (Entity _ Address {addressAddress = "30-50 Feral Hogs Rd"}))
+                                            }
+                               }
+                             )) -> True
+                 _ -> False)
+
+    itDb "can left join on nothing nested records" $ do
+        setup
+        records <- select $ do
+          from (table @User `leftJoin` myNestedMaybeRecordQuery `on` (do \(_ :& _) -> val False))
+        let sortedRecords = sortOn (\(Entity _ user :& _) -> getField @"userName" user) records
+        liftIO $ sortedRecords !! 0
+          `shouldSatisfy`
+          (\case (_ :& Nothing) -> True
+                 _ -> False)
+        liftIO $ sortedRecords !! 1
+          `shouldSatisfy`
+          (\case (_ :& Nothing) -> True
+                 _ -> False)
+
     itDb "can be used in a CTE" $ do
         setup
         records <- select $ do
@@ -271,9 +397,9 @@ testDeriveEsqueletoRecord = describe "deriveEsqueletoRecord" $ do
                                         , myAddress = (Just (Entity addr2 Address {addressAddress = "30-50 Feral Hogs Rd"}))
                                         }
                               )) -> True
-                 _ -> True)
+                 _ -> False)
 
-    itDb "can can handle joins on records with Nothing" $ do
+    itDb "can handle joins on records with Nothing" $ do
         setup
         records <- select $ do
           from
@@ -291,7 +417,7 @@ testDeriveEsqueletoRecord = describe "deriveEsqueletoRecord" $ do
                                         , myAddress = (Just (Entity addr2 Address {addressAddress = "30-50 Feral Hogs Rd"}))
                                         }
                               )) -> True
-                 _ -> True)
+                 _ -> False)
 
     itDb "can left join on nested records" $ do
         setup
@@ -312,7 +438,7 @@ testDeriveEsqueletoRecord = describe "deriveEsqueletoRecord" $ do
                                                                     , myAddress = (Just (Entity addr2 Address {addressAddress = "30-50 Feral Hogs Rd"}))
                                                                     }
                                               })) -> True
-                 _ -> True)
+                 _ -> False)
 
     itDb "can handle multiple left joins on the same record" $ do
         setup
@@ -331,7 +457,7 @@ testDeriveEsqueletoRecord = describe "deriveEsqueletoRecord" $ do
                                                                     , myAddress = (Just (Entity addr2 Address {addressAddress = "30-50 Feral Hogs Rd"}))
                                                                     }
                                               })) -> True
-                 _ -> True)
+                 _ -> False)
         liftIO $ sortedRecords !! 1
           `shouldSatisfy`
           (\case (_ :& _ :& Just (MyNestedRecord {myRecord = MyRecord {myName = "Rebecca", myAddress = Nothing}})) -> True
